import matplotlib.pyplot as plt
import numpy as np
from PIL import Image
import cv2
import random

def create_pascal_label_colormap():
    colormap = np.zeros((256, 3), dtype=int)
    ind = np.arange(256, dtype=int)
    for shift in reversed(range(8)):
        for channel in range(3):
            colormap[:, channel] |= ((ind >> channel) & 1) << shift
        ind >>= 3
    return colormap

def label_to_color_image(label):
    if label.ndim != 2:
        raise ValueError('Expect 2-D input label')
    colormap = create_pascal_label_colormap()
    if np.max(label) >= len(colormap):
        raise ValueError('label value too large.')
    return colormap[label]

def vis_segmentation(image, seg_map, name='demo.jpg', if_save=False, if_show=True):

    seg_image = label_to_color_image(seg_map).astype(np.uint8)
    plt.figure()
    plt.imshow(image)
    plt.imshow(seg_image, alpha=0.7)
    plt.axis('off')

    if if_save:
        plt.savefig("vis_{}".format(name))

    if if_show:
        plt.show()

def random_paste(im: np.ndarray, seg: np.ndarray):

    contours, _ = cv2.findContours(seg, cv2.RETR_TREE, cv2.CHAIN_APPROX_SIMPLE)
    if contours:
        cn_area = [cv2.contourArea(cn) for cn in contours]
        max_cont = contours[cn_area.index(np.max(cn_area))]

        xmin, ymin, w_, h_ = cv2.boundingRect(max_cont)
        xmax = xmin + w_
        ymax = ymin + h_

        seg_roi = seg[ymin:ymax, xmin:xmax]
        img_roi = im[ymin:ymax, xmin:xmax]

        roi = cv2.add(img_roi, np.zeros(np.shape(img_roi), dtype=np.uint8), mask=seg_roi)
        roi_h, roi_w = roi.shape[:2]

        bgh = 0
        bgw = 0
        while not ((roi_h < bgh) & (roi_w < bgw)):
            # bg_index = random.randint(0, len(self.hard_bglist) - 1)
            hard_im_path = 'xx'
            if input_type == 'rgb':
                bgm = cv2.imdecode(np.fromfile(hard_im_path, dtype=np.uint8), 1)
            else:
                bgm = cv2.imdecode(np.fromfile(hard_im_path, dtype=np.uint8), 0)

            bgh, bgw = bgm.shape[:2]
        
        tmp_list = [bgh / roi_h, bgw / roi_w]



# -*- coding: utf-8 -*-
# Created on 8月-31-21 10:21
# @site: https://github.com/moyans
# @author: moyan
import glob
import os
import pickle
import xml.etree.ElementTree as ET
from os import listdir, getcwd
from os.path import join

class VOC2YOLO:
    def __init__(self, classes=['person', 'car']):
        self.classes = classes

    def convert(self, size, box):
        dw = 1./(size[0])
        dh = 1./(size[1])
        x = (box[0] + box[1])/2.0 - 1
        y = (box[2] + box[3])/2.0 - 1
        w = box[1] - box[0]
        h = box[3] - box[2]
        x = x*dw
        w = w*dw
        y = y*dh
        h = h*dh
        return (x,y,w,h)

    def convert_annotation(self, xml_file, save_txt_path):

        out_file = open(save_txt_path, 'w')
        tree = ET.parse(xml_file)
        root = tree.getroot()
        size = root.find('size')
        w = int(size.find('width').text)
        h = int(size.find('height').text)

        for obj in root.iter('object'):
            difficult = obj.find('difficult').text
            cls = obj.find('name').text
            if cls not in self.classes or int(difficult)==1:
                continue
            cls_id = self.classes.index(cls)
            xmlbox = obj.find('bndbox')
            xmin = float(xmlbox.find('xmin').text) 
            xmax = float(xmlbox.find('xmax').text) 
            ymin = float(xmlbox.find('ymin').text)
            ymax = float(xmlbox.find('ymax').text)
            if xmax <= xmin: continue
            if ymax <= ymin: continue
            b = (xmin, xmax, ymin, ymax)
            bb = self.convert((w,h), b)
            out_file.write(str(cls_id) + " " + " ".join([str(a) for a in bb]) + '\n')

def main():
    classes = ['aeroplane', 'bicycle', 'bird', 'boat',
                'bottle', 'bus', 'car', 'cat', 'chair',
                'cow', 'diningtable', 'dog', 'horse',
                'motorbike', 'person', 'pottedplant',
                'sheep', 'sofa', 'train', 'tvmonitor']

    voc2yolo = VOC2YOLO(classes=classes)



if __name__ == '__main__':
    main()